@@ -3,6 +3,7 @@
 """
 import os
 from typing import Generator, Callable, Set
+from datetime import datetime, timedelta
 
 import requests
 
@@ -23,6 +24,7 @@ class CustomFalconReader:
         self.request_session: requests.Session = requests.Session()
         self.base_url: str = "https://api.falcon.io/"
         self.datasets: list = []
+        self.channel_id_index: int = None
 
     @retry_handler(exceptions=ConnectionError, total_tries=5)
     def _get_channel_ids(self) -> Set[str]:
@@ -52,17 +54,26 @@ class CustomFalconReader:
         backoff_method=os.environ.get("REQUEST_BACKOFF_METHOD", 0.01),
     )
     @retry_handler(exceptions=TimeoutError, total_tries=5, initial_wait=900)
+    @retry_handler(exceptions=ConnectionError, total_tries=5, initial_wait=900)
     def _content_by_channel_id(self, date: str, channel_id: str) -> list:
-
+        """Gets the content metrics by channel id
+        SEE: https://falconio.docs.apiary.io/#reference/channel-api/get-facebook-and-instagram-page-insights-metrics/get-content-metrics-by-channel-ids
+        :param: channel_id - integer
+        :param: date - string pushed as '2022-07-20'
+        :returns: list of dictionaries
+        """
         dataset: list = []
         endpoint_url: str = (
-            f"{self._config['endpoint']}?apikey={self._creds['api_key']}"
+            f"measure/api/v1/content/metrics?apikey={self._creds['api_key']}"
         )
         offset: int = 0
         limit = self._config.get("limit", 200)
         while True:
+            print(
+                f"INFO: channel id index: {self.channel_id_index}, channel id: {channel_id}, date: {date}, offset: {offset}."
+            )
             req = requests.Request(
-                method=self._config.get("method", "POST"),
+                method="POST",
                 url=f"{self.base_url}{endpoint_url}",
                 headers={"Content-Type": "application/json"},
                 json={
@@ -85,12 +96,18 @@ class CustomFalconReader:
             reason: str = response.reason
             if status_code == 200:
                 results: dict = response.json()
-                offset += limit
-                if len(results) == 0:
+                result_count = len(results)
+
+                if result_count == 0:  # or result_count < limit:
                     break  # we will exit the loop because no data is got back
+
+                if result_count < limit:
+                    offset += result_count
+                else:
+                    offset += limit
+
                 dataset.extend(results)
-                # if len(dataset) > 150:
-                #     break
+
             elif reason == "Not Found" and status_code == 404:
                 print(f"No data for channel id: {channel_id}, skipping.\n")
                 break
@@ -107,23 +124,35 @@ class CustomFalconReader:
         backoff_method=os.environ.get("REQUEST_BACKOFF_METHOD", 0.01),
     )
     @retry_handler(exceptions=TimeoutError, total_tries=5, initial_wait=900)
+    @retry_handler(exceptions=ConnectionError, total_tries=5, initial_wait=900)
     def _published_posts_by_channel_id(self, date: str, channel_id: str) -> list:
+        """Gets the published posts by channel id
+        SEE: https://falconio.docs.apiary.io/#reference/content-api/get-content-metrics-by-channel-ids
+        :param: channel_id - integer
+        :param: date - string pushed as '2022-07-20' but converted to isoformat '2022-07-20T00:00:00'
+        :returns: list of dictionaries
+        """
 
         dataset: list = []
-        endpoint_url: str = (
-            f"{self._config['endpoint']}?apikey={self._creds['api_key']}"
-        )
+        endpoint_url: str = f"publish/items?apikey={self._creds['api_key']}"
+        limit = self._config.get("limit", 200)
+        start_date = datetime.strptime(date, "%Y-%m-%d").isoformat()
+        end_date = (datetime.strptime(date, "%Y-%m-%d") + timedelta(days=1)).isoformat()
         while endpoint_url:
+            print(
+                f"INFO: channel id index: {self.channel_id_index}, channel id: {channel_id},  date: {date}, offset: {len(dataset)}."
+            )
             req = requests.Request(
-                method=self._config.get("method", "GET"),
+                method="GET",
                 url=f"{self.base_url}{endpoint_url}",
                 headers={"Content-Type": "application/json"},
-                json={
+                params={
                     "channels": [channel_id],
-                    "since": date,
-                    "until": date,
+                    "since": start_date,
+                    "until": end_date,
                     "networks": self._config["networks"],
                     "statuses": self._config.get("statuses", "published"),
+                    "limit": limit,
                 },
             )
             prepped = self.request_session.prepare_request(req)
@@ -138,7 +167,6 @@ class CustomFalconReader:
                 dataset.extend(items_data)
 
                 endpoint_url = results.get("next", {"href": None}).get("href")
-                # print(endpoint_url)
 
                 if len(items_data) == 0 or endpoint_url is None:
                     break
@@ -154,14 +182,14 @@ class CustomFalconReader:
         return dataset
 
     def _query_handler(self, date: str, channel_id: str) -> list:
-        endpoint = self._config.get("endpoint", None)
-        if not endpoint:
+        endpoint_name = self._config.get("endpoint_name", None)
+        if not endpoint_name:
             raise Exception(
-                "You need to specify endpoint in the configs i.e measure/api/v1/content/metrics or publish/items"
+                "You need to specify endpoint in the configs i.e content_metrics or published_posts"
             )
-        elif "content" in endpoint:
+        elif endpoint_name == "content_metrics":
             query = self._content_by_channel_id
-        elif "publish" in endpoint:
+        elif endpoint_name == "published_posts":
             query = self._published_posts_by_channel_id
 
         return query(date, channel_id)
@@ -184,13 +212,13 @@ class CustomFalconReader:
         ):
             payload: list = []
             for idx, channel_id in enumerate(channel_ids):
+                self.channel_id_index = idx
                 data = self._query_handler(channel_id=channel_id, date=date)
                 if data and len(data) > 0:
                     payload.extend(data)
 
             if len(payload) > 0:
                 yield {
-                    "endpoint_name": self._config["endpoint_name"],
                     "networks": self._config["networks"],
                     "date": date.replace("-", ""),
                     "data": payload,
